options(stringsAsFactors=FALSE)
library(ggplot2)
library(ggforce)
library(reshape2)

data = read.table("mp.all.txt", sep=",", na.strings = "NAN")
colnames(data) = c('pdbid', 'resolution', 'chainID', 'resNum', 'name', 'metal', 'occupancy', 'symmetry', 'totalDensity')

metals = data[which(data$metal != '0'),]
#length(unique(metals$pdbid)) / length(unique(data$pdbid))
#[1] 0.3752825

chainMedian <- read.table("f000.results.txt", sep=",", header=TRUE)
electronNum = apply(metals, 1, function(x) as.numeric(x[9]) / chainMedian$chainMedian[which(chainMedian$pdbid == x[1])[1]])
metals$electronNum = electronNum

#write.csv(metals, file="~/Desktop/metals.fullresults.txt")
#metals <- read.table("~/Desktop/metals.fullresults.txt", sep=",", header = TRUE, na.strings = "NAN")
  
metalList = c('ZN', 'MG', 'CA','FE', 'NA', 'MN', 'K', 'CU', 'NI', 'CO', 'CD', 'HG', 'PT', 'MO', 'AL', 'V',
             'BA', 'SR', 'RU', 'CS', 'W', 'YB', 'AU', 'Y', 'LI', 'PB', 'GD', 'TL', 'RB', 'SM', 'IR', 'PR',
             'RH', 'EU', 'PD', 'AG', 'OS', 'LU', 'HO', 'TB', 'CR', 'GA', 'LA', 'SB', 'CE', 'ER', 'IN', 'BI', 
             'DY', 'BE', 'ZR', 'SN', 'HF', 'TA', 'RE', 'PA', 'U')

temp = metals$electronNum[! is.na(metals$electronNum)]
std2cutoff = (sum(temp ^ 2)/length(temp) ) ^ 0.5 * 2
temp = metals$electronNum[metals$resolution <= 2.5 & metals$occupancy >= 0.9 & ! is.na(metals$electronNum) & metals$electronNum < std2cutoff]
electronCutoff = (sum(temp ^ 2)/length(temp) ) ^ 0.5
#electronCutoff
#19.3543

sumTable = NULL
for (metal in metalList) {
  oneMetal = metals[metals$metal == metal, ] 
  result = c(metal,
             length(unique(oneMetal$pdbid)),
             nrow(oneMetal),
             sum(oneMetal$resolution<=2.5),
             sum(oneMetal$occupancy>=0.9),
             sum(table(oneMetal$symmetry)[c("-1", "0")]),
             sum(oneMetal$electronNum < electronCutoff, na.rm = TRUE),
             sum(oneMetal$resolution<=2.5 & oneMetal$occupancy>=0.9 & oneMetal$symmetry != 1 & oneMetal$electronNum < electronCutoff, na.rm = TRUE)
  )
  sumTable = rbind(sumTable, result)
}

metal = 'CU' # 'ZN', 'CA', 'FE', 'MN', and 'CU'
tiff(paste(tolower(metal),".resolution.tiff", sep=''), units="in", width=5, height=5, res=400, compression = 'lzw', type = "cairo")
df = metals[metals$metal == metal & metals$resolution >= 0,]
ggplot(df, aes(x = resolution)) + geom_histogram(binwidth = 0.1) + 
  labs(x = '') + xlim(0, 10) +  theme(text = element_text(size=15)) +
  geom_vline(xintercept = 2.5, color = "red", size=0.2) + 
  annotate("text",  x=Inf, y = Inf, label = metal, vjust=1, hjust=1, size=7)
dev.off()

tiff(paste(tolower(metal),".electrons.tiff", sep=''), units="in", width=5, height=5, res=400, compression = 'lzw', type = "cairo")
df = metals[metals$metal == metal & ! is.na(metals$electronNum),]
ggplot(df, aes(x = electronNum)) + geom_histogram(binwidth = 1) + 
  xlim(0, 300) + ylim(0,2200) + labs(x = '') + theme(text = element_text(size=15)) +
  geom_vline(xintercept = electronCutoff, color = "red", size=0.2) + 
  annotate("text",  x=Inf, y = Inf, label = metal, vjust=1, hjust=1, size=7)
dev.off()



#### Overall
tiff(paste("all.resolution.tiff", sep=''), units="in", width=5, height=5, res=400, compression = 'lzw', type = "cairo")
ggplot(metals, aes(x = resolution)) + geom_histogram(binwidth = 0.1) + 
  xlim(0, 10) + labs(x = expression(paste("Resolution (", ring(A), ")"))) + theme(text = element_text(size=15)) +
  geom_vline(xintercept = 2.5, color = "red", size=0.2) + 
  annotate("text",  x=Inf, y = Inf, label = 'ALL', vjust=1, hjust=1, size=7)
dev.off()

tiff(paste("all.electrons.tiff", sep=''), units="in", width=5, height=5, res=400, compression = 'lzw', type = "cairo")
ggplot(metals, aes(x = electronNum)) + geom_histogram(binwidth = 1) + 
  xlim(0, 300) + ylim(0,17000) + labs(x = expression(paste("Electrons of Discrepancy (", "e"^"-", ")"))) + theme(text = element_text(size=15)) +
  geom_vline(xintercept = electronCutoff, color = "red", size=0.2) + 
  annotate("text",  x=Inf, y = Inf, label = 'All', vjust=1, hjust=1, size=7)
dev.off()


## Table S1. high quality metal list
highquality = metals[which(metals$resolution<=2.5 & metals$occupancy>=0.9 & metals$symmetry != 1 & metals$electronNum < electronCutoff), ]
write.csv(highquality, file='TableS1.txt')


## Figure S1. Background 

bg = read.table("background.txt", sep=",")
colnames(bg) = c('pdbid', 'totalDensity', 'volume', 'sphereDensity')

metalBg = bg[which(bg$pdbid %in% metals$pdbid), ]
electrons = apply(metalBg, 1, function(x) as.numeric(x[4]) / chainMedian$chainMedian[which(chainMedian$pdbid == x[1])[1]])
metalBg$electronNum = electrons

tiff(paste("FigureS1.tiff", sep=''), units="in", width=5, height=5, res=400, compression = 'lzw', type = "cairo")
ggplot(metalBg, aes(x = electronNum)) + geom_histogram(binwidth = 0.2) + 
  labs(x = expression(paste("Electrons of Discrepancy (", "e"^"-", ")"))) + theme(text = element_text(size=15)) +
  geom_vline(xintercept = electronCutoff, color = "red", size=0.2) + 
  annotate("text",  x=Inf, y = Inf, label = 'Background', vjust=1, hjust=1)
dev.off()

median(metals$electronNum, na.rm = TRUE)
mean(metals$electronNum, na.rm = TRUE)

median(electrons, na.rm = TRUE)
mean(electrons, na.rm = TRUE)


## Figure S2. metal clusters
clusterData = read.table("metalCluster.txt", sep=",", na.strings = "NAN")
colnames(clusterData) = c('pdbid', 'chainID', 'resNum', 'metal', 'minMetalDist')
hist(clusterData$minMetalDist, 100)
summary(clusterData$minMetalDist)

cluster = clusterData[clusterData$minMetalDist<=3,]
clusterList = paste(cluster$pdbid, cluster$chainID, cluster$resNum, cluster$metal, sep =".")
metalL = paste(metals$pdbid, metals$chainID, metals$resNum, metals$metal, sep =".")

metalClusters = metals[which(metalL %in% clusterList), ]
mean(metalClusters$electronNum, na.rm = TRUE)

tiff(paste("FigureS2.tiff", sep=''), units="in", width=5, height=5, res=400, compression = 'lzw', type = "cairo")
ggplot(metalClusters, aes(x = electronNum)) + geom_histogram(binwidth = 1) + 
  xlim(0, 300) + labs(x = expression(paste("Electrons of Discrepancy (", "e"^"-", ")"))) + theme(text = element_text(size=15)) +
  geom_vline(xintercept = electronCutoff, color = "red", size=0.2) + 
  annotate("text",  x=Inf, y = Inf, label = 'Metal Clusters', vjust=1, hjust=1, size=7)
dev.off()


## Fitered out percentage
temp = metalClusters$electronNum[which(! is.na(metalClusters$electronNum))] 
length(temp[temp>electronCutoff])  / length(temp)

temp = metals$electronNum[which(! is.na(metals$electronNum))] 
length(temp[temp>electronCutoff])  / length(temp)



